-- this is the program that shall generate very very gooood msuic

function love.load(Args)
	FS = love.filesystem
	math.randomseed(os.time())

	-- variables                                   blargle
	TrackCount = 1
	TrackLength = 0
	TrackLengthMin = 60
	TrackLengthMax = 240
	TrackLengthDefault = 120
	SampleRate = 44100
	BitDepth = 16

	-- parse command line options like "--tracks=50"
	for k,v in pairs(Args) do
		ParseOption(v,"tracks","TrackCount")
		ParseOption(v,"length","TrackLength")
		ParseOption(v,"length-min","TrackLengthMin")
		ParseOption(v,"length-max","TrackLengthMax")
		ParseOption(v,"sample-rate","SampleRate")
		ParseOption(v,"bit-depth","BitDepth")
		assert((BitDepth==8 or BitDepth==16), "Bit depth must be 8 or 16")
	end
	print("TrackCount = "..tostring(TrackCount))
	print("TrackLength = "..tostring(TrackLength))
	print("TrackLengthMin = "..tostring(TrackLengthMin))
	print("TrackLengthMax = "..tostring(TrackLengthMax))
	print("SampleRate = "..tostring(SampleRate))
	print("BitDepth = "..tostring(BitDepth))

	-- TO AVOID CONFUSION:
	-- "Samples" will refer to a (BitDepth)-bit integer representing a single sample in a peice of sound data
	-- "Cuts" will refer to large portions of samples taken from another audio source
	-- here we create the user directories if they dont already exist
	if not FS.getInfo("Cuts","directory") then
		love.filesystem.createDirectory("Cuts")
	end
	if not FS.getInfo("Tracks","directory") then
		love.filesystem.createDirectory("Tracks")
	end
	if not FS.getInfo("Effects","directory") then
		love.filesystem.createDirectory("Effects")
	end

	-- get a list of cuts and effects
	AvailableCuts = FS.getDirectoryItems("Cuts")
	AvailableEffects = FS.getDirectoryItems("Effects")
	assert(#AvailableCuts>0,"There are no audio files or something in the Cuts folder")
	if not (#AvailableEffects>0) then
		print("[WARNING] There are no effects to use from the effects folder, only stock effects will be used")
	end

	do -- load in the effects
		local EffectList = {
			LoadEffect("StockEffects/RandomChopping"),
			LoadEffect("StockEffects/PitchControl")
		}
		for k,v in pairs(AvailableEffects) do
			local Effect = LoadEffect("Effect/"..v,v)
			if Effect then table.insert(EffectList,Effect) end
		end
		AvailableEffects = EffectList
	end
	for k,v in pairs(AvailableEffects) do print(v,v.Name) end

	for i=1,TrackCount,1 do
		local Length = (TrackLength == 0) and math.random(TrackLengthMin,TrackLengthMax) or TrackLengthDefault
		GenerateTrack(Length)
	end
end

-- parses parameters passed via command line
function ParseOption(Arg,Option,Var,IsString)
	if Arg:match("^--"..Option.."=") then
		local Val = string.sub(Arg,#Option+4,#Arg)
		if not IsString then
			Val = assert(tonumber(Val),Option.." ("..Val..") is not a valid number")
		end
		_G[Var] = Val
	end
end

function LoadEffect(M,FileName)
	local FileName = FileName or "?"
	local Effect = require(M)
	if not Effect.Name then Effect.Name = "unknown" end
	if not Effect.Description then Effect.Description = "No description provided." end
	if not Effect.Operate then
		print("[WARNING] Could not load effect \""..Effect.Name.."\" ("..FileName..")  because it does not have an Operate function, skipping...")
		return
	end
	print("Loaded effect \""..Effect.Name.."\"")
	--table.insert(AvailableEffects,Effect)
	return Effect
end

function ApplyScrewups(_Cut)
	local Cut = _Cut
	if type(Cut)=="string" then
		Cut = love.sound.newSoundData(_Cut)
	end

	for i=1,math.random(1,3),1 do
		local CanCut = (Cut:getDuration() <= 10) and 2 or 1 -- make sure to exclude the random chopping stock effect if the cut it already very short
		local h = math.random(CanCut,#AvailableEffects)
		local ChosenEffect = AvailableEffects[h]
		print("        Applying effect \""..ChosenEffect.Name.."\"..")
		Cut = ChosenEffect:Operate(Cut)
	end
	return Cut
end

-- the star of the show, the track generator!!!!!!
function GenerateTrack(Length)
	print("Generating a "..tostring(Length).." second long track...")

	local Track = love.sound.newSoundData(
		SampleRate * Length,
		SampleRate,
		BitDepth,
		2 -- mono is for pussies smh, stereo ftw
	)

	local function ChooseCut()
		return "Cuts/"..AvailableCuts[math.random(1,#AvailableCuts)]
	end
	local BaseCut = ChooseCut()
	local UsedCuts = {} -- note: UsedCuts[1] will be the "base" cut
	for I=1,math.random(2,2+(Length/60)),1 do --what!
		local ChosenCut = ChooseCut()
		print("    Picking cut \""..ChosenCut.."\"")
		table.insert(UsedCuts,ChosenCut)--love.sound.newSoundData(ChosenCut))
	end

	for k,v in pairs(UsedCuts) do
		local Position = math.random(1,SampleRate * Length)
		local ThisCut = ApplyScrewups(v)

		-- TODO: overlay the cut onto the track

		print("Applying the cut to the track..")
	end
end
